from fastapi import FastAPI, HTTPException, APIRouter, Depends,Request
from pydantic import BaseModel
import chargebee
import os
import logging
from utils.db import mongo_database

# Initialize logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

# Initialize Stripe with your secret key
API_KEY = os.getenv("CHARGEBEE_SECRET_KEY")
SITE = os.getenv("CHARGEBEE_SITE")

chargebee.configure("{API_KEY}","{SITE}")
# Initialize FastAPI and MongoDB client
app = FastAPI()

chargebee_session_router = csr = APIRouter()

class CheckoutSessionRequest(BaseModel):
    price_id: str
    email: str
    quantity: int
    unit_price: int

@csr.post("/create-checkout-session")
async def create_checkout_session(request: CheckoutSessionRequest):
    logger.info("Received request to create checkout session")
    # logger.info(f"Request data: {request}")
    data=await request.json()
    # print(data)
    email=data['email']
    price_id=data['price_id']
    quantity=data['quantity']
    unit_price=data['unit_price']
    
    # Fetch the customer ID from MongoDB
    try:
        user_data = mongo_database["Users"].find_one({"email": email})
        if not user_data:
            logger.error(f"User with email {email} not found")
            raise HTTPException(status_code=404, detail="User not found")
    except Exception as e:
        logger.error(f"Error fetching user data: {e}")
        raise HTTPException(status_code=500, detail="Error fetching user data")

    customer_id = user_data.get("stripe_customer_id")

    logger.info(f"Fetched customer ID: {customer_id}")
    success_url=os.getenv("PAYMENT_SUCCESS_URL")
    cancel_url=os.getenv("PAYMENT_CANCEL_URL")
    # Create a Stripe Checkout Session
    try:
        # checkout_session = chargebee.HostedPage.checkout_existing_for_items(
        #     customer=customer_id,
        #     payment_method_types=["card"],
        #     line_items=[
        #         {
        #             "price": price_id,
        #             "quantity": 1,
        #         },
        #     ],
        #     mode="subscription",
        #     success_url=success_url,
        #     cancel_url=cancel_url,
        # )

        checkout_session = chargebee.HostedPage.checkout_existing_for_items({
            "subscription" : {"id" : customer_id},
            "subscription_items" : [
                {
                    "item_price_id" : price_id,
                    "quantity" : quantity,
                    "unit_price" : unit_price
                }]
            })
        
        hosted_page = checkout_session.hosted_page
        logger.info(f"Created checkout session: {hosted_page['id']}")
        return {"id": checkout_session["id"]}
    
    except Exception as e:
        logger.error(f"Error creating checkout session: {e}")
        raise HTTPException(status_code=500, detail="Error creating checkout session")



from typing import List

from fastapi import APIRouter, Depends, HTTPException, Request, status
from fastapi.responses import StreamingResponse
from llama_index.core.chat_engine.types import BaseChatEngine
from llama_index.core.llms import ChatMessage
from llama_index.core.llms import MessageRole

from llama_index.agent.openai import OpenAIAgent
from llama_index.llms.openai import OpenAI
from pydantic import BaseModel

from app.engine import get_chat_engine

# Create an instance of APIRouter to handle chat-related routes
chat_router = r = APIRouter()

# Define a Pydantic model for Message data
class _Message(BaseModel):
    """
    The message data consisting of the role and content of the message.
    role: The role of the message. For eg. USER or SYSTEM.
    """
    role: MessageRole
    content: str

# Define a Pydantic model for ChatData
class _ChatData(BaseModel):
    messages: List[_Message]

# Endpoint to handle chat requests
@r.post("")
async def chat(
    request: Request,
    data: _ChatData,
    chat_engine: BaseChatEngine = Depends(get_chat_engine),
):
    """
    Send messages and receive chat responses in a streaming manner.

    :param request: Incoming request.
    :type request: Request
    :param data: Data containing messages to be sent.
    :type data: _ChatData
    :param chat_engine: Chat engine dependency.
    :type chat_engine: BaseChatEngine
    :raises HTTPException 400: If no messages provided or if last message is not from user.
    :return: Streaming response containing chat responses.
    :rtype: StreamingResponse
    """
    # check preconditions and get last message
    if len(data.messages) == 0:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="No messages provided",
        )
    lastMessage = data.messages.pop()
    if lastMessage.role != MessageRole.USER:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="Last message must be from user",
        )
    # convert messages coming from the request to type ChatMessage
    messages = [
        ChatMessage(
            role=m.role,
            content=m.content,
        )
        for m in data.messages
    ]

    prompt = lastMessage.content 

    # query chat engine
    response = await chat_engine.astream_chat(prompt, messages)
    
    # stream response
    async def event_generator():
        # print(str(response.async_response_gen()))
        async for token in response.async_response_gen():
            # If client closes connection, stop sending events
            if await request.is_disconnected():
                break
            yield token

    return StreamingResponse(event_generator(), media_type="text/plain")



import stripe
import os
from datetime import datetime, timedelta
from bson import ObjectId
from fastapi import HTTPException
from utils.db import mongo_database

import logging

logger = logging.getLogger("uvicorn")

stripe.api_key = os.getenv('STRIPE_SECRET_KEY')

def find_user_by_email(email: str):
    user = mongo_database["Users"].find_one({"email": email})
    if not user:
        raise HTTPException(status_code=404, detail="User not found")
    return user

def find_user_by_id(user_id: str):
    user = mongo_database["Users"].find_one({"_id": ObjectId(user_id)})
    if not user:
        raise HTTPException(status_code=404, detail="User not found")
    return user

def calculate_end_date(start_date: datetime, frequency: str) -> datetime:
    if frequency == "monthly":
        return start_date + timedelta(days=30)
    elif frequency == "yearly":
        return start_date + timedelta(days=365)
    else:
        raise ValueError("Invalid frequency. Must be 'monthly' or 'yearly'.")
    
def calculate_prorate_amount(subscription):
    try:
        sample_subscription_item = {
                "id": subscription["items"]["data"][0]["id"],
                "plan": subscription["items"]["data"][0]["plan"]["id"],
                "quantity": 0,
            }

        upcoming_prorated_invoice = stripe.Invoice.upcoming(
            customer=subscription["customer"],
            subscription=subscription["id"],
            subscription_items=[sample_subscription_item],
        )

        # Calculate the prorated amount
        prorated_amount = 0
        for invoice_item in upcoming_prorated_invoice["lines"]["data"]:
            if invoice_item["type"] == "invoiceitem" and invoice_item["amount"] < 0:
                prorated_amount = abs(invoice_item["amount"])
                break

        # logger.info(f"Calculated prorated amount: {prorated_amount}")
        return prorated_amount
    except Exception as e:
        logging.error("Error While calculating Refund")
        raise HTTPException(status_code=400, detail=str(e))

def update_subscription_details(user, subscription):
    # Extract subscription details from the event data
    plan_id = subscription['items']['data'][0]['plan']['id']
    current_period_start = subscription['current_period_start']
    current_period_end = subscription['current_period_end']

    # Retrieve plan details from Stripe using product_id and plan_id
    plan = stripe.Plan.retrieve(plan_id, expand=['product'])

    # Extract plan type and frequency from the retrieved plan
    plan_type = plan['product']['name']
    frequency = plan['interval']

    # Convert timestamps to ISO format strings
    start_date = datetime.fromtimestamp(current_period_start).isoformat() + "Z"
    end_date = datetime.fromtimestamp(current_period_end).isoformat() + "Z"
    
    previous_subscription_id=user["stripe_subscription_id"]
    
    stripe.api_key=os.getenv("STRIPE_SECRET_KEY")
    
    try:
    
        previous_subscription = stripe.Subscription.retrieve(previous_subscription_id)

        prorated_amount=calculate_prorate_amount(previous_subscription)

        latest_invoice_id = previous_subscription["latest_invoice"]
        if latest_invoice_id:
            invoice = stripe.Invoice.retrieve(latest_invoice_id)
            # logger.info(invoice)
            # unused_amount = invoice["amount_due"] - invoice["amount_paid"]
            if prorated_amount > 0:
                # Create a refund for the unused amount
                stripe.Refund.create(
                    charge=invoice["charge"],
                    amount=prorated_amount,
                )
            logger.info(f"Refunded amount: {prorated_amount}")
            
        canceled_Subscription=stripe.Subscription.cancel(
            previous_subscription_id,
            prorate=False
            )
        logger.info(f"Canceled subscription: {previous_subscription_id}")

        # Update data to be set in MongoDB
        update_data = {
            "stripe_subscription_id": subscription['id'],
            "start_date": start_date,
            "end_date": end_date,
            "user_companies_count": 0,
            "user_jobs_count": 0,
            "plan": plan_type,  # Set plan type retrieved from Stripe
            "user_contacts_count": 0,
            "user_emails_count": 0,
            "updated_at": datetime.now()
        }

        # Update MongoDB with subscription details
        mongo_database["Users"].update_one(
            {"_id": user["_id"]},
            {"$set": update_data}
        )
        return start_date, end_date, plan_type, frequency
    
    except Exception as e:
        raise HTTPException(status_code=400, detail=str(e))
    
    # return start_date, end_date, plan_type, frequency



# from llama_index.core import ServiceContext
# import os
# import logging
# from logging_config import LOGGING_CONFIG
# logging.config.dictConfig(LOGGING_CONFIG)

# from app.engine.constants import CHUNK_SIZE, CHUNK_OVERLAP
# from llama_index.embeddings.openai import OpenAIEmbedding
# from llama_index.core.node_parser import SentenceSplitter
# from llama_index.llms.openai import OpenAI
# from llama_index.core import Settings
# from llama_index.core.constants import DEFAULT_TEMPERATURE

# def create_service_context():
#     logger = logging.getLogger("uvicorn")
#     print("Initializing the OpenAI configurations.......")
#     logger.info("Initializing the OpenAI configurations.......")
#     max_tokens = os.getenv("LLM_MAX_TOKENS")
#     config = {
#         "model": os.getenv("MODEL"),
#         "temperature": float(os.getenv("LLM_TEMPERATURE", DEFAULT_TEMPERATURE)),
#         "max_tokens": int(max_tokens) if max_tokens is not None else None,
#     }
#     Settings.llm = OpenAI(**config)
#     dimensions = os.getenv("EMBEDDING_DIM")
#     config = {
#         "model": os.getenv("EMBEDDING_MODEL"),
#         "dimensions": int(dimensions) if dimensions is not None else None,
#     }
#     Settings.embed_model = OpenAIEmbedding(**config)
#     return Settings


# stripe_webhook.py

from fastapi import APIRouter, HTTPException, Request
import stripe
import os
from utils.db import mongo_database
from utils.subscription_management import update_subscription_details

webhook_router = APIRouter()
endpoint_secret = os.getenv('STRIPE_WEBHOOK_SECRET')

@webhook_router.post("/")
async def stripe_webhook(request: Request):
    payload = await request.body()
    sig_header = request.headers.get('stripe-signature')

    try:
        event = stripe.Webhook.construct_event(payload, sig_header, endpoint_secret)
    except (ValueError, stripe.error.SignatureVerificationError) as e:
        raise HTTPException(status_code=400, detail=str(e))
    
    try:
        if event['type'] == 'customer.subscription.updated':
            handle_subscription_event(event['data']['object'])
    except Exception as e:
        raise HTTPException(status_code=400, detail=str(e)) 
    
    return {"status": "success"}

def handle_subscription_event(subscription):
    stripe_customer_id = subscription['customer']
    user = mongo_database["Users"].find_one({"stripe_customer_id": stripe_customer_id})
    if user:
        update_subscription_details(user, subscription)



import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.base import MIMEBase
from email import encoders
from urllib.parse import unquote
from pydantic import BaseModel, EmailStr
from google.oauth2.credentials import Credentials
from google.auth.exceptions import RefreshError
from googleapiclient.discovery import build
from googleapiclient.errors import HttpError
from msal import ConfidentialClientApplication
import requests
from app.engine import get_chat_engine
from llama_index.core.chat_engine.types import BaseChatEngine
from google.auth.transport.requests import Request
from fastapi import APIRouter, HTTPException, Request, Depends
from dotenv import load_dotenv
from typing import List, Optional
import base64
import os
load_dotenv()
import logging

email_router = e = APIRouter()

class TokenData(BaseModel):
    accessToken: str
    refreshToken: str
    clientId: str
    clientSecret: str
    idToken: str
    userEmail: str
    scope: str

class EmailPayload(BaseModel):
    from_email: Optional[EmailStr] = None
    to_email: Optional[List[EmailStr]] = None
    cc: Optional[List[EmailStr]] = None
    subject: Optional[str] = None
    body: Optional[str] = None
    attachments: Optional[List[str]] = None

@e.post("/send/admin")    
async def send_email_by_admin_account(emailPayload: EmailPayload):
    from_email = os.getenv("ADMIN_EMAIL")
    if not from_email:
        logging.error("Admin email is not set in environment variables")
        return False

    to_email = emailPayload.to_email
    subject = emailPayload.subject
    body = emailPayload.body
    attachments = emailPayload.attachments

    comma_separated_emails = ",".join(to_email) if to_email else ""
    if not comma_separated_emails:
        logging.error("Recipient email addresses are empty or malformed")
        return False

    msg = MIMEMultipart()
    msg['From'] = from_email
    msg['To'] = unquote(comma_separated_emails)

    if subject:
        msg['Subject'] = subject
    else:
        logging.warning("Email subject is empty")

    if body:
        msg.attach(MIMEText(body, 'plain'))
    else:
        logging.warning("Email body is empty")

    # Attach files if any
    if attachments:
        for attachment_path in attachments:
            try:
                with open(attachment_path, "rb") as attachment:
                    part = MIMEBase('application', 'octet-stream')
                    part.set_payload(attachment.read())
                    encoders.encode_base64(part)
                    filename = os.path.basename(attachment_path)
                    part.add_header('Content-Disposition', f'attachment; filename={filename}')
                    msg.attach(part)
            except FileNotFoundError:
                logging.error(f"Attachment file not found: {attachment_path}")
            except PermissionError:
                logging.error(f"Permission denied for attachment file: {attachment_path}")
            except Exception as e:
                logging.error(f"Unexpected error attaching file {attachment_path}: {e}")
    
    try:
        s = smtplib.SMTP('smtp.gmail.com', 587)
        s.starttls()
        s.login(from_email, os.getenv("ADMIN_EMAIL_PASSWORD"))
        s.sendmail(from_email, unquote(comma_separated_emails), msg.as_string())
        s.quit()
        logging.info("Email sent successfully through admin email")
        return True
    except smtplib.SMTPAuthenticationError:
        logging.error("SMTP authentication failed. Check ADMIN_EMAIL and ADMIN_EMAIL_PASSWORD")
    except smtplib.SMTPConnectError as e:
        logging.error(f"SMTP connection error: {e}")
    except smtplib.SMTPRecipientsRefused:
        logging.error(f"All recipients were refused: {comma_separated_emails}")
    except smtplib.SMTPException as e:
        logging.error(f"SMTP error occurred: {e}")
    except Exception as e:
        logging.error(f"Unexpected error while sending email: {e}")
    return False
        
def create_message(emailPayload):
    sender = emailPayload.from_email
    to = emailPayload.to_email
    subject = emailPayload.subject
    message_text = emailPayload.body

    # Convert the list of to_email addresses to a comma-separated string
    to_emails = ', '.join(to) if to else ''

    message = MIMEMultipart()
    message['to'] = to_emails
    message['from'] = sender
    message['subject'] = subject

    # Attach the message text as HTML
    msg = MIMEText(message_text, 'html')
    message.attach(msg)

    raw_message = base64.urlsafe_b64encode(message.as_string().encode('utf-8'))
    return {'raw': raw_message.decode('utf-8')}

def send_message(service, user_id, message):
    try:
        message = service.users().messages().send(userId=user_id, body=message).execute()
        logging.info('Message Id: %s' % message['id'])
        return message
    except HttpError as error:
        logging.error('An error occurred: %s' % error)
        return None

@e.post("/send")
async def send_email_from_user_email(tokenData: dict, emailPayload: EmailPayload):
    def send_message_gmail(service, user_id, message):
        # Send email via Gmail API
        service.users().messages().send(userId=user_id, body=message).execute()

    def send_message_microsoft(access_token, payload):
        # Ensure payload is converted to a dictionary or JSON
        url = "https://graph.microsoft.com/v1.0/me/sendMail"
        headers = {
            "Authorization": f"Bearer {access_token}",
            "Content-Type": "application/json"
        }

        # Convert EmailPayload to a dictionary if it's not already
        if isinstance(payload, EmailPayload):
            payload_dict = {
                "message": {
                    "subject": payload.subject,
                    "body": {
                        "contentType": "HTML",  # Ensure content type is HTML
                        "content": payload.body
                    },
                    "toRecipients": [{"emailAddress": {"address": email}} for email in payload.to_email]
                }
            }
            if payload.cc:
                payload_dict["message"]["ccRecipients"] = [
                    {"emailAddress": {"address": email}} for email in payload.cc
                ]
            if payload.attachments:
                payload_dict["message"]["attachments"] = payload.attachments
        else:
            payload_dict = payload  # Assume it's already in a serializable format

        response = requests.post(url, headers=headers, json=payload_dict)
        if response.status_code not in (200, 202):
            raise RuntimeError(f"Error sending email via Microsoft Graph API: {response.text}")

    try:
        # Determine the provider based on scopes
        if "https://www.googleapis.com/auth/gmail.send" in tokenData.get("scope", ""):
            # Handle Gmail account
            SCOPES = tokenData["scope"].split()
            creds = Credentials(
                token=tokenData["accessToken"],
                refresh_token=tokenData["refreshToken"],
                token_uri="https://oauth2.googleapis.com/token",
                client_id=tokenData["clientId"],
                client_secret=tokenData["clientSecret"],
                scopes=SCOPES
            )
            try:
                gmail_service = build('gmail', 'v1', credentials=creds)
                message = create_message(emailPayload)
                send_message_gmail(gmail_service, 'me', message)
                return {"status_code": 200, "message": "Email sent successfully."}
            except RefreshError:
                # Refresh token logic for Gmail
                creds.refresh(Request())
                tokenData["accessToken"] = creds.token
                tokenData["refreshToken"] = creds.refresh_token
                gmail_service = build('gmail', 'v1', credentials=creds)
                message = create_message(emailPayload)
                send_message_gmail(gmail_service, 'me', message)
                return {"status_code": 200, "message": "Email sent successfully after token refresh."}

        elif "Mail.Send" in tokenData.get("scope", ""):
            # Handle Microsoft account
            access_token = tokenData["accessToken"]
            refresh_token = tokenData["refreshToken"]
            client_id = tokenData["clientId"]
            client_secret = tokenData["clientSecret"]
            authority = tokenData.get("authority", "https://login.microsoftonline.com/common")

            try:
                send_message_microsoft(access_token, emailPayload)
                return {"status_code": 200, "message": "Email sent successfully."}
            except RuntimeError:
                # Refresh token logic for Microsoft
                app = ConfidentialClientApplication(
                    client_id, authority=authority, client_credential=client_secret
                )
                result = app.acquire_token_by_refresh_token(refresh_token, scopes=["Mail.Send"])
                if "access_token" in result:
                    tokenData["accessToken"] = result["access_token"]
                    send_message_microsoft(result["access_token"], emailPayload)
                    return {"status_code": 200, "message": "Email sent successfully after token refresh."}
                else:
                    raise RuntimeError("Failed to refresh Microsoft token.")

        else:
            raise ValueError("Unsupported email provider or missing scope in tokenData.")

    except Exception as e:
        logging.error("Unexpected error while sending email", exc_info=True)
        raise RuntimeError(f"Unexpected error: {str(e)}") from e

@e.get("/generate")
async def generate_personalised_email(
    company_name:str,
    person_name: str,
    person_summary: str,
    title: str,
    chat_engine: BaseChatEngine = Depends(get_chat_engine),
):
    prompt = "You are an expert recruiter and co-pilot for recruitment industry. "
    prompt += "Help generate a Email based on Job Title, Person Summary and Person Name to be sent to the potential candidate. " 
    prompt+= "Company Name: " + company_name +"\n"
    
    if(person_name!=""):
      prompt += "Person Name: " + str(person_name) + "\n"
      
    prompt += "Person Summary:" + str(person_summary) + "\n"
    prompt += "Job Title:" + str(title) + "\n"
    
    prompt += "Try to Write like this: Hi Based on your description your profile is being shortlisted/rejeected etc. Try to Write in about 150 words. Do not Add Any Types Of Salutations. At Ending Just Write Recruiting Team and There Company Name"
    response=chat_engine.chat(prompt)
    return response.response


@e.get("/generate/summary")
async def generate_summary(
    job_title: str,
    person_summary: str
):
    chat_engine: Optional[BaseChatEngine] = None
    try:
        # Validate inputs
        if not job_title.strip():
            raise ValueError("Job title cannot be empty.")
        if not person_summary.strip():
            raise ValueError("Person summary cannot be empty.")

        # Prepare the prompt
        prompt = (
            "You are an expert recruiter and co-pilot for the recruitment industry. "
            f"Job Title: {job_title}\n"
            f"Person Summary: {person_summary}\n"
            "Summarise person's experience and expertise based on the given summary "
            "in the context of an interview mail. Personalise the content as if you are "
            "writing an email to the candidate to show them as the best fit. Do not write "
            "full email content but just a summary of the candidate's experience with "
            "regard to the given title. Try to summarise in about 50 words."
        )

        # Resolve chat_engine if not provided
        if chat_engine is None:
            chat_engine = get_chat_engine()

        # Call the chat engine
        response = chat_engine.chat(prompt)
        
        # Ensure the response is valid
        if not response or not hasattr(response, "response"):
            raise ValueError("Chat engine did not return a valid response.")

        return response.response

    except ValueError as ve:
        raise HTTPException(status_code=400, detail=str(ve))
    except Exception as e:
        raise HTTPException(status_code=500, detail="An error occurred while generating the summary.")


# Utility function to replace placeholders in the email template
async def populate_template(template: str, person: dict, job_title: str) -> str:
    try:
        # Validate inputs
        if not isinstance(template, str) or not template.strip():
            raise ValueError("Template must be a non-empty string.")
        if not isinstance(person, dict):
            raise ValueError("Person must be a dictionary.")
        if not isinstance(job_title, str) or not job_title.strip():
            raise ValueError("Job title must be a non-empty string.")

        # Fetch values with defaults
        first_name = person.get("first_name", "Candidate")
        current_company = person.get("experiences", [{}])[0].get("company", "Company")
        person_job_title = person.get("occupation", "Job Title")
        summary = person.get("summary", "skills and experience")

        # Generate the brief summary
        brief_summary: Optional[str] = await generate_summary(job_title, summary)

        if not brief_summary:
            raise ValueError("Failed to generate a brief summary for the candidate.")

        # Replace placeholders in the template
        populated_template = (
            template.replace("{{First Name}}", first_name)
            .replace("{{Current Company}}", current_company)
            .replace("{{Current Job Title}}", person_job_title)
            .replace("{{Client Company}}", "our company")
            .replace("{*Brief mention of the candidate’s relevant skills.*}", brief_summary)
            .replace("{{Client Job Title}}", job_title)
        )

        return populated_template

    except ValueError as ve:
        raise HTTPException(status_code=400, detail=str(ve))
    except Exception as e:
        raise HTTPException(status_code=500, detail="An error occurred while populating the template.")



from fastapi import APIRouter, HTTPException
from pydantic import BaseModel
from typing import List, Optional
from bson import ObjectId
from enum import Enum
from utils.db import mongo_database

status_router = s = APIRouter()

# MongoDB Setup
status_collection = mongo_database["statuses"]

class StatusValue(str, Enum):
    NOT_CONTACTED = "NOT_CONTACTED"
    CONTACTED = "CONTACTED"
    SCREENING_CALL = "SCREENING_CALL"
    REJECTED = "REJECTED"
    ON_HOLD = "ON_HOLD"
    HIRED = "HIRED"

class Status(BaseModel):
    status_label: str
    status_value: StatusValue

class StatusUpdate(BaseModel):
    ids: List[str]
    status: StatusValue

@s.put("/bulk")
async def update_bulk_statuses(status_update: StatusUpdate):
    object_ids = [ObjectId(id) for id in status_update.ids]
    result =  status_collection.update_many(
        {"_id": {"$in": object_ids}},
        {"$set": {"status_value": status_update.status}}
    )
    if result.modified_count == 0:
        raise HTTPException(status_code=404, detail="No statuses updated")
    return {"message": f"{result.modified_count} statuses updated"}

@s.post("/", response_model=Status)
async def create_status(status: Status):
    status_dict = status.dict()
    result = status_collection.insert_one(status_dict)
    status_dict["_id"] = str(result.inserted_id)
    return status_dict

@s.get("/", response_model=List[Status])
async def get_statuses():
    statuses = status_collection.find().to_list(1000)
    for status in statuses:
        status["_id"] = str(status["_id"])
    return statuses

@status_router.get("/statuses/label/{status_value}")
async def get_status_label(status_value: StatusValue):
    status_document =  status_collection.find_one({"status_value": status_value})
    if not status_document:
        raise HTTPException(status_code=404, detail="Status not found")
    return {"status_label": status_document["status_label"]}




from fastapi import APIRouter, HTTPException, Body
from pymongo import MongoClient
from utils.db import mongo_database
from typing import List
import stripe
import os

from datetime import datetime
from app.models import PlanData, PlanUpdater

# Create an instance of APIRouter to handle plan-related routes
plan_router = APIRouter()
stripe.api_key = os.getenv("STRIPE_SECRET_KEY")

@plan_router.post("/")
async def create_plan(plan_data: PlanData):
    # Instantiate PlanUpdater with connection details
    updater = PlanUpdater(mongo_database, "Plan")

    plan_data_dict = plan_data.dict()
    plan_data_dict["created_at"] = datetime.now()
    plan_data_dict["updated_at"] = datetime.now()
    
    # Create the product in Stripe
    # stripe_product = stripe.Product.create(
    #     name=plan_data.plan_type,
    #     description=plan_data.description,
    #     metadata={
    #         "max_companies_per_user": plan_data.max_companies_per_user,
    #         "max_jobs": plan_data.max_jobs,
    #         "max_resumes_per_job": plan_data.max_resumes_per_job,
    #         "max_searches_per_job": plan_data.max_searches_per_job,
    #         "max_total_search": plan_data.max_total_search,
    #         "max_emails_per_user": plan_data.max_emails_per_user,
    #         "max_mobile_contacts_per_user": plan_data.max_mobile_contacts_per_user
    #     }
    # )

    # Create the monthly price in Stripe
    # monthly_price = stripe.Price.create(
    #     unit_amount=plan_data.monthly_price,
    #     currency="usd",
    #     recurring={"interval": "month"},
    #     product=stripe_product.id
    # )

    # Create the yearly price in Stripe
    # yearly_price = stripe.Price.create(
    #     unit_amount=plan_data.yearly_price,
    #     currency="usd",
    #     recurring={"interval": "year"},
    #     product=stripe_product.id
    # )
    
    # plan_data_dict["monthly_price_id"]= monthly_price.id
    # plan_data_dict["yearly_price_id"]= yearly_price.id
        
    plan_id = updater.create_plan(plan_data_dict)

    return {
        "plan_id": plan_id,
        # "stripe_product_id": stripe_product.id,
        # "monthly_price_id": monthly_price.id,
        # "yearly_price_id": yearly_price.id,
        "message": "Plan created successfully"
    }

@plan_router.put("/{plan_id}")
async def update_plan(plan_id: str, plan_data: PlanData):
    # Instantiate PlanUpdater with connection details
    updater = PlanUpdater(mongo_database, "Plan")

    # Construct updates dictionary with provided parameters
    updates = plan_data.dict(exclude_unset=True)
    updates["updated_at"] = datetime.now()

    # Update the plan details in MongoDB
    updater.update_plan(plan_id, updates)

    # Fetch the current plan details from MongoDB
    current_plan = updater.collection.find_one({"plan_id": plan_id})

    if not current_plan or "stripe_product_id" not in current_plan:
        raise HTTPException(status_code=404, detail="Plan not found or Stripe product ID missing")

    # Update the product in Stripe
    stripe.Product.modify(
        current_plan["stripe_product_id"],
        name=plan_data.plan_type,
        description=plan_data.description,
        metadata={
            "max_companies_per_user": plan_data.max_companies_per_user,
            "max_jobs": plan_data.max_jobs,
            "max_resumes_per_job": plan_data.max_resumes_per_job,
            "max_searches_per_job": plan_data.max_searches_per_job,
            "max_total_search": plan_data.max_total_search,
            "max_emails_per_user": plan_data.max_emails_per_user,
            "max_mobile_contacts_per_user": plan_data.max_mobile_contacts_per_user
        }
    )

    return {"message": "Plan details updated successfully"}

# New endpoint to get all plans
@plan_router.get("/")
async def get_plans():
    updater = PlanUpdater(mongo_database, "Plan")
    plans = updater.get_plans()
    if not plans:
        raise HTTPException(status_code=404, detail="No plans found")
    return plans
